# 3DE4.script.name:	Export Distortion to After Effects Mesh Warp...
# 3DE4.script.version: v1.1
# 3DE4.script.gui:	Main Window::3DE4::File::Export
# 3DE4.script.comment:	Exports a jsx file which creates compositions in After Effects and applies animation presets to them.
# 3DE4.script.comment:	Two animation presets are exportet to remove/apply lens distortion via mesh grid

import sys
if not ("%s/sys_data/py_vl_sdv"%(tde4.get3DEInstallPath()) in sys.path): 
	sys.path.append("%s/sys_data/py_vl_sdv"%(tde4.get3DEInstallPath()))

from vl_sdv import *
import struct

def _export_ae_mesh_warp_toggle_overscan(requester,widget,action):
	flag = tde4.getWidgetValue(requester,widget)
	tde4.setWidgetSensitiveFlag(requester, "img_overscan_width",flag)
	tde4.setWidgetSensitiveFlag(requester, "img_overscan_height",flag)
	
def float_to_hex(f):
    return hex(struct.unpack('<I', struct.pack('<f', f))[0])


def concat_bytearray(a,b):
	for s in b:
		a.append(s)
	return a

def create_list_chunk(nkeyframes):
	chunk = bytearray([0x6C,0x69,0x73,0x74,0x6C,0x68,0x64,0x33,0x00,0x00,0x00,0x34,0x00,0xD0,0x0B,0xEE,0x33,0x04,0x1D,0x18,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x94,0xE7,0xEC,0x01,0xD4,0xE8,0xEC,0x01,0x0C,0xAF,0xEB,0x01])
	
	chunk[20:24] = struct.pack(">I",nkeyframes)
	return chunk
	
def create_ldat_chunk(frames, fps):
	chunk = bytearray(['l','d','a','t'])
	s = len(frames) * 60
	chunk = concat_bytearray(chunk,struct.pack(">I",s))
	for f in frames:
		timecode = int(f * fps) << 8
		
		chunk = concat_bytearray(chunk,struct.pack(">I",timecode))
		chunk = concat_bytearray(chunk,struct.pack(">I",0x01010001))
		chunk = concat_bytearray(chunk,struct.pack(">I",2))
		chunk = concat_bytearray(chunk,struct.pack(">I",0))
		chunk = concat_bytearray(chunk,struct.pack(">d",0))
		chunk = concat_bytearray(chunk,struct.pack(">d",1))
		chunk = concat_bytearray(chunk,struct.pack(">d",1))
		chunk = concat_bytearray(chunk,struct.pack(">d",1))
		chunk = concat_bytearray(chunk,struct.pack(">d",1))
		chunk = concat_bytearray(chunk,struct.pack(">f",0))
	return chunk

def create_aRbp_header(col,row):
	chunk = bytearray(['a','R','b','p'])
	s = row*col*4*10 + 12+8
	chunk = concat_bytearray(chunk,struct.pack(">I",s))
	chunk = concat_bytearray(chunk,struct.pack(">I",row))
	chunk = concat_bytearray(chunk,struct.pack(">I",col))
	chunk = concat_bytearray(chunk,bytearray([0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0D,0x0D]))
	
	return chunk

def add_float_to_aRbp(aRbp,val1,val2):
	aRbp = concat_bytearray(aRbp,struct.pack(">ff",val1,val2))
	return aRbp


	
def create_LIST_chunk(chunks):
	chunk = bytearray(['L','I','S','T'])
	s = 0
	for c in chunks:
		s = s + len(c)
	l = struct.pack(">I",s)
	chunk = concat_bytearray(chunk,l)
	for c in chunks:
		chunk = concat_bytearray(chunk,c)
	return chunk

def create_tdmn_end_group_chunk():
	return bytearray([0x74,0x64,0x6D,0x6E,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x47,0x72,0x6F,0x75,0x70,0x20,0x45,0x6E,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00])
def create_main_chunks():
	return bytearray([0x52,0x49,0x46,0x58,0x00,0x00,0x40,0x94,0x46,0x61,0x46,0x58,0x68,0x65,0x61,0x64,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x4a,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x40,0x70,0x62,0x65,0x73,0x63,0x62,0x65,0x73,0x6f,0x00,0x00,0x00,0x38,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x64,0x00,0x00,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x00,0x04,0x2b,0x0a,0x00,0x04,0x2b,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0xac,0x74,0x64,0x73,0x70,0x74,0x64,0x6f,0x74,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0xff,0x74,0x64,0x70,0x6c,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x02,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0x40,0x74,0x64,0x73,0x69,0x74,0x64,0x69,0x78,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0xff,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x45,0x66,0x66,0x65,0x63,0x74,0x20,0x50,0x61,0x72,0x61,0x64,0x65,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0x40,0x74,0x64,0x73,0x69,0x74,0x64,0x69,0x78,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x0a,0x4d,0x65,0x73,0x68,0x20,0x57,0x61,0x72,0x70,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0x64,0x74,0x64,0x73,0x70,0x74,0x64,0x6f,0x74,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0xff,0x74,0x64,0x70,0x6c,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x01,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0x40,0x74,0x64,0x73,0x69,0x74,0x64,0x69,0x78,0x00,0x00,0x00,0x04,0xff,0xff,0xff,0xff,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x45,0x6e,0x64,0x20,0x6f,0x66,0x20,0x70,0x61,0x74,0x68,0x20,0x73,0x65,0x6e,0x74,0x69,0x6e,0x65,0x6c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x3e,0xf2,0x73,0x73,0x70,0x63,0x66,0x6e,0x61,0x6d,0x00,0x00,0x00,0x30,0x4d,0x65,0x73,0x68,0x20,0x57,0x61,0x72,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x04,0xc8,0x70,0x61,0x72,0x54,0x70,0x61,0x72,0x6e,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x05,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x61,0x72,0x64,0x00,0x00,0x00,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x31,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x61,0x72,0x64,0x00,0x00,0x00,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x52,0x6f,0x77,0x73,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x61,0x72,0x64,0x00,0x00,0x00,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x43,0x6f,0x6c,0x75,0x6d,0x6e,0x73,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x1f,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x35,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x61,0x72,0x64,0x00,0x00,0x00,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x51,0x75,0x61,0x6c,0x69,0x74,0x79,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x0a,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x0a,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x61,0x72,0x64,0x00,0x00,0x00,0x94,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x06,0x00,0x28,0x00,0x00,0x00,0x0b,0x44,0x69,0x73,0x74,0x6f,0x72,0x74,0x69,0x6f,0x6e,0x20,0x4d,0x65,0x73,0x68,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00,0xd2,0x04,0x00,0x00,0x30,0x6a,0xc3,0x30,0x50,0x98,0x1f,0x23,0xef,0xbe,0xad,0xde,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x61,0x52,0x62,0x70,0x00,0x00,0x00,0xb4,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0x0d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xaa,0xaa,0x9f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0xaa,0xaa,0x9f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x3e,0xaa,0xaa,0x9f,0x3f,0x2a,0xaa,0x9f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x2a,0xaa,0x9f,0x3e,0xaa,0xaa,0x9f,0x3f,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x2a,0xaa,0x9f,0x3f,0x80,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x80,0x00,0x00,0x3f,0x2a,0xaa,0x9f,0x3f,0x80,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x39,0xde,0x74,0x64,0x67,0x70,0x74,0x64,0x73,0x62,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x09,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x0a,0x4d,0x65,0x73,0x68,0x20,0x57,0x61,0x72,0x70,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0xda,0x74,0x64,0x62,0x73,0x74,0x64,0x73,0x62,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x03,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x01,0x00,0x00,0x74,0x64,0x62,0x34,0x00,0x00,0x00,0x7c,0xdb,0x99,0x00,0x01,0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x64,0x00,0x3f,0x1a,0x36,0xe2,0xeb,0x1c,0x43,0x2d,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0x64,0x61,0x74,0x00,0x00,0x00,0x28,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x70,0x69,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x0d,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x31,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0xf2,0x74,0x64,0x62,0x73,0x74,0x64,0x73,0x62,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x01,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x05,0x52,0x6f,0x77,0x73,0x00,0x00,0x74,0x64,0x62,0x34,0x00,0x00,0x00,0x7c,0xdb,0x99,0x00,0x01,0x00,0x01,0x00,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x64,0x00,0x3f,0x1a,0x36,0xe2,0xeb,0x1c,0x43,0x2d,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0x64,0x61,0x74,0x00,0x00,0x00,0x28,0x40,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x75,0x6d,0x00,0x00,0x00,0x08,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x75,0x4d,0x00,0x00,0x00,0x08,0x40,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0xf4,0x74,0x64,0x62,0x73,0x74,0x64,0x73,0x62,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x01,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x08,0x43,0x6f,0x6c,0x75,0x6d,0x6e,0x73,0x00,0x74,0x64,0x62,0x34,0x00,0x00,0x00,0x7c,0xdb,0x99,0x00,0x01,0x00,0x01,0x00,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x64,0x00,0x3f,0x1a,0x36,0xe2,0xeb,0x1c,0x43,0x2d,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0x64,0x61,0x74,0x00,0x00,0x00,0x28,0x40,0x1c,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x75,0x6d,0x00,0x00,0x00,0x08,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x75,0x4d,0x00,0x00,0x00,0x08,0x40,0x3f,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x35,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x00,0xf4,0x74,0x64,0x62,0x73,0x74,0x64,0x73,0x62,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x01,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x08,0x51,0x75,0x61,0x6c,0x69,0x74,0x79,0x00,0x74,0x64,0x62,0x34,0x00,0x00,0x00,0x7c,0xdb,0x99,0x00,0x01,0x00,0x01,0x00,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x64,0x00,0x3f,0x1a,0x36,0xe2,0xeb,0x1c,0x43,0x2d,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0x64,0x61,0x74,0x00,0x00,0x00,0x28,0x40,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x75,0x6d,0x00,0x00,0x00,0x08,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x75,0x4d,0x00,0x00,0x00,0x08,0x40,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x74,0x64,0x6d,0x6e,0x00,0x00,0x00,0x28,0x41,0x44,0x42,0x45,0x20,0x4d,0x45,0x53,0x48,0x20,0x57,0x41,0x52,0x50,0x2d,0x30,0x30,0x30,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4c,0x49,0x53,0x54,0x00,0x00,0x02,0x28,0x74,0x64,0x62,0x73,0x74,0x64,0x73,0x62,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x01,0x74,0x64,0x73,0x6e,0x00,0x00,0x00,0x10,0x44,0x69,0x73,0x74,0x6f,0x72,0x74,0x69,0x6f,0x6e,0x20,0x4d,0x65,0x73,0x68,0x00,0x74,0x64,0x62,0x34,0x00,0x00,0x00,0x7c,0xdb,0x99,0x00,0x01,0x00,0x06,0x00,0x01,0x00,0x06,0x00,0x07,0x00,0x00,0x64,0x00,0x3f,0x1a,0x36,0xe2,0xeb,0x1c,0x43,0x2d,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,0xf0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x04,0x01,0x00,0x00,0x00,0xa4,0x96,0x68,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00]) 	


def ae_fib(frames):
	v1 = 0
	v2 = 1
	i = 0
	while 4*(v2 - 1) < int(frames):
		i += 1
		f = v1+v2
		v1 = v2
		v2 = f
		
	return [v2-1,4*(v2-1)]

def write_ae_mesh_warp_preset(filename,grid_res_x,grid_res_y,frames,frame_offset, dist_mode,grid, pf_offset):
	cam = tde4.getCurrentCamera()
	fps = tde4.getCameraFPS(cam)
	[xa,xb,ya,yb] = tde4.getCameraFOV(cam)
	
	f = open(filename,"wb")
	a = create_main_chunks()
	t_width = 1.0 / float(grid_res_x) / 3.0
	t_height = 1.0 / float(grid_res_y) / 3.0
	
	aRbs_chunk = bytearray(['a','R','b','s'])
	pf = 0
	for frame in range(0, frames):
		aRbp_chunk = create_aRbp_header(grid_res_x+1,grid_res_y+1)
		aRbs_chunk = concat_bytearray(aRbs_chunk,aRbp_chunk)
		
		for y in reversed(range(0,grid_res_y+1)):
			for x in range(0,grid_res_x+1):
			
				r = [t_width,0]
				l = [-t_width,0.0]
				o = [0.0,t_height]
				u=[0.0,-t_height]
	
				jmat = mat2d(tde4.getDistortion2DJacobianMatrix(cam,frame+1,[grid[frame][y][x][0],grid[frame][y][x][1]]))
				
				if dist_mode != 1:
					jmat = jmat.invert()
				o = jmat*vec2d(o)
				u = jmat*vec2d(u)
				r = jmat*vec2d(r)
				l = jmat*vec2d(l)
				
				pointx = grid[frame][y][x][2]*(xb-xa) 
				pointy = 1.0 - (grid[frame][y][x][3]*(yb-ya))
				ta_lx = pointx + l[0]*(xb-xa)
				ta_ly = (pointy - l[1]*(yb-ya))
				ta_rx = pointx + r[0]*(xb-xa)
				ta_ry = (pointy - r[1]*(yb-ya))
				ta_ox = pointx + o[0]*(xb-xa)
				ta_oy = (pointy - o[1]*(yb-ya))
				ta_ux = pointx + u[0]*(xb-xa)
				ta_uy =  (pointy - u[1]*(yb-ya))
			
				aRbs_chunk = add_float_to_aRbp(aRbs_chunk,pointx,pointy)
				aRbs_chunk = add_float_to_aRbp(aRbs_chunk,ta_ox,ta_oy)
				aRbs_chunk = add_float_to_aRbp(aRbs_chunk,ta_rx,ta_ry)
				aRbs_chunk = add_float_to_aRbp(aRbs_chunk,ta_ux,ta_uy)
				aRbs_chunk = add_float_to_aRbp(aRbs_chunk,ta_lx,ta_ly)
			pf = pf + 1
			cont		= tde4.updateProgressRequester(pf + pf_offset,"Write AE Mesh Warp Preset...")
			if not cont: break

	
	list_chunk = create_list_chunk(frames)
	ldat_chunk = create_ldat_chunk([x+frame_offset for x in range(1,frames+1)], 100.0/fps)
	LIST_chunk = create_LIST_chunk([list_chunk,ldat_chunk])
	LIST_chunk2 = create_LIST_chunk([aRbs_chunk])
	end_chunk = create_tdmn_end_group_chunk()
	a = concat_bytearray(a,LIST_chunk)
	a = concat_bytearray(a,LIST_chunk2)
	a = concat_bytearray(a,end_chunk)
	
	l = len(LIST_chunk)
	l2 = len(LIST_chunk2) + len(end_chunk)
	a[4:8] = struct.pack(">I",3152+l+l2)
	a[40:44] = struct.pack(">I",3116+l+l2)
	a[422:426] = struct.pack(">I",2734+l+l2)
	a[1722:1726] = struct.pack(">I",1434+l+l2)
	a[2984:2988] = struct.pack(">I",172+l)
	a[826:830] = struct.pack(">I",grid_res_x)
	a[1030:1034] = struct.pack(">I",grid_res_y)
	
	lup = {7 : 28, 11 : 38, 13 : 42, 19 : 51}
	a[2261] = struct.pack("b",lup[grid_res_x])
	a[2561] = struct.pack("b",lup[grid_res_y])
	
	v1,v2 = ae_fib(frames)
	
	a[3192:3196] = struct.pack(">I",int(v1))
	a[3208:3212] = struct.pack(">I",int(v2))
	
	
	f.write(a)
	f.close()
def write_jsx(filename,apply_preset_path,remove_preset_path,overscan_w,overscan_h,frames):
	f = open(filename,"w")
	cam = tde4.getCurrentCamera()
	lens = tde4.getCameraLens(cam)
	
	width = tde4.getCameraImageWidth(cam)
	height= tde4.getCameraImageHeight(cam)
	
	overscan_width = width * (1.0 + overscan_w)
	overscan_height = height * (1.0 + overscan_h)
	fps = tde4.getCameraFPS(cam)
	pixel_aspect = tde4.getLensPixelAspect(lens)
	
	f.write('var footage_rem_comp = app.project.items.addComp("footage_remove", %i,%i,%f,%i,%i);\n'%(width,height,pixel_aspect,frames,fps))
	f.write('var footage_app_comp = app.project.items.addComp("footage_apply", %i,%i,%f,%i,%i);\n'%(overscan_width,overscan_height,pixel_aspect,frames,fps))
	f.write('var apply_comp = app.project.items.addComp("apply", %i,%i,%f,%i,%i);\n'%(width,height,pixel_aspect,frames,fps))
	f.write('var remove_comp = app.project.items.addComp("remove", %i,%i,%f,%i,%i);\n'%(overscan_width,overscan_height,pixel_aspect,frames,fps))
	f.write('var footage_apply = apply_comp.layers.add(footage_app_comp);\n')
	f.write('var footage_remove = remove_comp.layers.add(footage_rem_comp);\n')
	
	f.write('f = new File("%s");\n'%apply_preset_path)
	f.write('if(!f.exists)f = File.openDialog ("Open %s", ":*.ffx", false);\n'%apply_preset_path)
	f.write('footage_apply.applyPreset(f);\n')
	f.write('f = new File("%s");\n'%remove_preset_path)
	f.write('if(!f.exists)f = File.openDialog ("Open %s", ":*.ffx", false);\n'%remove_preset_path)
	f.write('footage_remove.applyPreset(f);\n')
	
	f.close()
	
def create_filepathes(path):
	path_jsx = path
	if not path_jsx[-4:] == ".jsx":
		path_jsx += ".jsx"
	path_remove_preset = path_jsx[:-4] + "_remove.ffx"
	path_apply_preset  = path_jsx[:-4] + "_apply.ffx"
	
	return [path_remove_preset, path_apply_preset, path_jsx]

def _export_ae_mesh_warp(path, grid_res_x, grid_res_y,startframe, overscan_width,overscan_height):
	cam = tde4.getCurrentCamera()

	width = tde4.getCameraImageWidth(cam)
	height= tde4.getCameraImageHeight(cam)

	lens = tde4.getCameraLens(cam)
	pixel_aspect = tde4.getLensPixelAspect(lens)

	#calc tile size (image border is 2 px for clamping)
	tiles_w = (width)/float(grid_res_x) 
	tiles_h = (height)/float(grid_res_y)
	frames = tde4.getCameraNoFrames(cam)
	
	#if(not tde4.getLensDynamicDistortionFlag(lens)):
	#	frames = 1
	grid_remove = []
	grid_apply = []
	tde4.postProgressRequesterAndContinue("Export AE Mesh Warp Preset...", "...", (grid_res_y+1)*2*frames + frames, "Cancel")
	
	#create 2D array with entrys [posx,posy,undist x, undist y] all in unitspace
	
	
	overscan_x = overscan_width*0.5
	overscan_y = overscan_height*0.5
	pf = 0
	for frame in range(1, frames+1):
		fr_apply = []
		fr_remove = []
		for y in range(0,grid_res_y+1):
			row_remove = []
			row_apply = []
			for x in range(0,grid_res_x+1):
				p1 = [(x*tiles_w)/width,(y*tiles_h)/height]
				p2 = tde4.removeDistortion2D(cam,frame,p1)
				row_remove.append([p1[0],p1[1],p2[0],p2[1]])
				
				p1[0] = -overscan_x + p1[0]*(overscan_x*2 + 1.0)
				p1[1] = -overscan_y + p1[1]*(overscan_y*2 + 1.0)
				p2 = tde4.applyDistortion2D(cam,frame,p1)
				row_apply.append([p1[0],p1[1],(p2[0] + overscan_x)/(overscan_x*2 + 1.0),(p2[1] + overscan_y)/(overscan_y*2 + 1.0)])
				
			
			fr_remove.append(row_remove)
			fr_apply.append(row_apply)
		pf = pf + 1
		cont		= tde4.updateProgressRequester(pf,"Prepare AE Mesh Warp Preset...")
		if not cont: break
		grid_remove.append(fr_remove)
		grid_apply.append(fr_apply)
	
	[path_remove_preset, path_apply_preset, path_jsx] = create_filepathes(path)
	
	write_ae_mesh_warp_preset(path_remove_preset,grid_res_x,grid_res_y,frames,startframe - 1, 1,grid_remove,pf)
	pf += (grid_res_y+1)*frames
	write_ae_mesh_warp_preset(path_apply_preset,grid_res_x,grid_res_y,frames,startframe - 1, 0,grid_apply,pf)
	write_jsx(path_jsx,path_apply_preset,path_remove_preset,overscan_width,overscan_height,frames)
	
	

try:
	_ae_ld_export_requester
except (ValueError,NameError,TypeError):
	_ae_ld_export_requester	= tde4.createCustomRequester()
	tde4.addFileWidget(_ae_ld_export_requester,"file_browser","Exportfile...","*.jsx")

	
	tde4.addOptionMenuWidget(_ae_ld_export_requester,"grid_width","Mesh Warp Resolution","7","11","13","19")
	tde4.setWidgetLinks(_ae_ld_export_requester,"grid_width","file_browser","", "file_browser", "")
	tde4.setWidgetOffsets(_ae_ld_export_requester,"grid_width",30,0,5,0)
	tde4.setWidgetAttachModes(_ae_ld_export_requester,"grid_width","ATTACH_AS_IS","ATTACH_NONE","ATTACH_AS_IS","ATTACH_AS_IS")
	tde4.setWidgetSize(_ae_ld_export_requester, "grid_width", 80,20)
	tde4.addOptionMenuWidget(_ae_ld_export_requester,"grid_height","X","7","11","13","19")
	tde4.setWidgetOffsets(_ae_ld_export_requester,"grid_height",25,0,0,0)
	tde4.setWidgetLinks(_ae_ld_export_requester,"grid_height","grid_width","", "grid_width", "")
	tde4.setWidgetAttachModes(_ae_ld_export_requester,"grid_height","ATTACH_WIDGET","ATTACH_NONE","ATTACH_OPPOSITE_WIDGET","ATTACH_NONE")
	tde4.setWidgetSize(_ae_ld_export_requester, "grid_height", 80,20)
	tde4.addTextFieldWidget(_ae_ld_export_requester,"startframe","Startframe","1")

	tde4.addToggleWidget(_ae_ld_export_requester,"use_overscan","Overscan",0)

	tde4.addTextFieldWidget(_ae_ld_export_requester,"img_overscan_width","Overscan Resolution")
	tde4.setWidgetLinks(_ae_ld_export_requester,"img_overscan_width","use_overscan","", "use_overscan", "")
	tde4.setWidgetOffsets(_ae_ld_export_requester,"img_overscan_width",30,0,5,0)
	tde4.setWidgetAttachModes(_ae_ld_export_requester,"img_overscan_width","ATTACH_AS_IS","ATTACH_NONE","ATTACH_AS_IS","ATTACH_AS_IS")
	tde4.setWidgetSize(_ae_ld_export_requester, "img_overscan_width", 80,20)
	tde4.addTextFieldWidget(_ae_ld_export_requester,"img_overscan_height","X")
	tde4.setWidgetOffsets(_ae_ld_export_requester,"img_overscan_height",25,0,0,0)
	tde4.setWidgetLinks(_ae_ld_export_requester,"img_overscan_height","img_overscan_width","", "img_overscan_width", "")
	tde4.setWidgetAttachModes(_ae_ld_export_requester,"img_overscan_height","ATTACH_WIDGET","ATTACH_NONE","ATTACH_OPPOSITE_WIDGET","ATTACH_NONE")
	tde4.setWidgetSize(_ae_ld_export_requester, "img_overscan_height", 80,20)
	tde4.setWidgetCallbackFunction(_ae_ld_export_requester,"use_overscan","_export_ae_mesh_warp_toggle_overscan")
	tde4.setWidgetSensitiveFlag(_ae_ld_export_requester, "img_overscan_width",0)
	tde4.setWidgetSensitiveFlag(_ae_ld_export_requester, "img_overscan_height",0)

fov = tde4.getCameraFOV(tde4.getCurrentCamera())
tde4.setWidgetValue(_ae_ld_export_requester,"img_overscan_width",str(int(tde4.getCameraImageWidth(tde4.getCurrentCamera()))))
tde4.setWidgetValue(_ae_ld_export_requester,"img_overscan_height",str(int(tde4.getCameraImageHeight(tde4.getCurrentCamera()))))

tde4.setWidgetValue(_ae_ld_export_requester, "grid_width", "2")
tde4.setWidgetValue(_ae_ld_export_requester, "grid_height", "2")
	
tde4.setWidgetValue(_ae_ld_export_requester,"startframe",str(tde4.getCameraFrameOffset(tde4.getCurrentCamera())))

ret	= tde4.postCustomRequester(_ae_ld_export_requester,"Export Distortion to AE Mesh Warp Preset...",540,230,"Ok","Cancel")
if ret==1:
	path	= tde4.getWidgetValue(_ae_ld_export_requester,"file_browser")
	grid_res = {1 : 7, 2 : 11, 3 : 13, 4 : 19}
	
	grid_res_x = tde4.getWidgetValue(_ae_ld_export_requester,"grid_width")
	grid_res_y = tde4.getWidgetValue(_ae_ld_export_requester,"grid_height")
	startframe = int(tde4.getWidgetValue(_ae_ld_export_requester,"startframe"))
	
	img_overscan_width  = float(tde4.getWidgetValue(_ae_ld_export_requester,"img_overscan_width"))/float(tde4.getCameraImageWidth(tde4.getCurrentCamera())) - 1.0
	img_overscan_height = float(tde4.getWidgetValue(_ae_ld_export_requester,"img_overscan_height"))/float(tde4.getCameraImageHeight(tde4.getCurrentCamera())) - 1.0
	if tde4.getWidgetValue(_ae_ld_export_requester,"use_overscan") != 1:
		img_overscan_width = 0
		img_overscan_height= 0
		
	if path != None:
		_export_ae_mesh_warp(path,grid_res[grid_res_x],grid_res[grid_res_y],startframe,img_overscan_width,img_overscan_height )
